import Config from './config'
import Store from './store'
import Render from './render'

import { merge } from 'lodash'

export default {
	install(Vue) {
		Vue.mixin({
			created() {
				if(this.$store && !this.$parent) {
					
					this.$store.registerModule(Config.namespace, Store, {
						preserveState: Config.preserve
					})

					this.$store.commit(`${Config.namespace}/label`, merge(Store.state.data, this.$options.label))

					var _render = (key) => {
						let mediaRender = ['@image', '@video', '@widget', '@info']
						let rule = key.split('.')[0]
						if(mediaRender.includes(rule)) {
							key = key.split(`${rule}.`)[1]
							if(rule == '@image') return [key, 'image']
							if(rule == '@info') return [key, 'info']
							if(rule == '@video') return [key, 'video']
							if(rule == '@widget') { return [key, 'widget'] }
						} else {
							return [key, 'language']
						}
					}

					var __render = (store, key, value) => {
						let _r = _render(key)
						let _k = _r[0]
						let _t = _r[1]
						return Render[_t](store, _k, value)
					}

					Vue.prototype.$render = (key, value = null) => {
						if (key) return __render(this.$store, key, value)
					}

					Vue.prototype.$source = (key, value = null) => {
						if(key == 'language' && !Config.language.supported.includes(value)) {
							console.warn(`The language '${key}' is not supported!`)
							return false
						}
						this.$store.commit(`${Config.namespace}/status`, { type: key, value: value })
					}

					Vue.directive('render', {
						bind: function(el, binding, vNode) {
							let bind = __render(vNode.context.$store, el.innerHTML, binding.expression)
							if(el.innerHTML.includes('@image')) bind = `<img src="${bind}">`
							el.setAttribute('render-ref', el.innerHTML)
							el.innerHTML = bind
						},
						update: function(el, binding, vNode) {
							let bind = __render(vNode.context.$store, el.getAttribute('render-ref'), binding.expression)
							if(el.getAttribute('render-ref').includes('@image')) bind = `<img src="${bind}">`
							el.innerHTML = bind
						}
					})

				}
			},
			updated() {
				for (let event in this.$store.getters[`${Config.namespace}/events`]) {
					if (document.querySelector(`#${event}`)) {
						document.querySelector(`#${event}`).onclick = this.$store.getters[`${Config.namespace}/events`][event]
					}
				}
			}
		})
	}
}